@@ -1,9 +1,6 @@
 #![forbid(unsafe_code)]
 #![deny(clippy::all)]
 
-use slog::{debug, error, Logger};
-use sloggers::null::NullLoggerBuilder;
-use sloggers::Build;
 use std::{iter, mem};
 use yozuk_sdk::model::*;
 use yozuk_sdk::prelude::*;
@@ -36,7 +33,6 @@ pub struct Yozuk {
     labelers: Vec<Box<dyn Labeler>>,
     commands: Vec<Option<CommandCache>>,
     redirections: Vec<(Vec<Token>, Vec<String>)>,
-    logger: Logger,
 }
 
 impl Yozuk {
@@ -49,14 +45,12 @@ impl Yozuk {
     }
 
     pub fn get_commands(&self, tokens: &[Token], streams: &[InputStream]) -> Vec<CommandArgs> {
-        debug!(self.logger, "{:?}", tokens);
-
         let filter = |(redirect, _): &&(Vec<Token>, Vec<String>)| {
             redirect.len() == tokens.len()
                 && redirect
                     .iter()
-                    .map(|token| token.as_utf8())
-                    .zip(tokens.iter().map(|token| token.as_utf8()))
+                    .map(|token| token.as_str())
+                    .zip(tokens.iter().map(|token| token.as_str()))
                     .all(|(a, b)| yozuk_helper_english::normalized_eq(a, [b], 0))
         };
 
@@ -128,7 +122,7 @@ impl Yozuk {
         commands: Vec<CommandArgs>,
         streams: &mut [InputStream],
         i18n: Option<&I18n>,
-    ) -> Result<Output, Vec<Output>> {
+    ) -> Result<Vec<Output>, Vec<Output>> {
         let commands = commands.into_iter().filter_map(|args| {
             self.model
                 .get_index(&args.args[0])
@@ -136,22 +130,36 @@ impl Yozuk {
                 .map(|cmd| (args, &cmd.command))
         });
 
+        let mut primary = None;
+        let mut results = Vec::new();
         let mut errors = Vec::new();
         for (args, command) in commands {
             let name = args.args[0].clone();
             match command.run(args, streams, i18n.unwrap_or(&self.i18n)) {
-                Ok(result) => return Ok(result),
+                Ok(result) => {
+                    if result.mode == OutputMode::Primary {
+                        if primary.is_none() {
+                            primary = Some(result);
+                        }
+                    } else {
+                        results.push(result);
+                    }
+                }
                 Err(err) => errors.push(err.into_output(name)),
             }
         }
 
-        Err(errors)
+        if errors.is_empty() {
+            Ok(primary.into_iter().chain(results).collect())
+        } else {
+            Err(errors)
+        }
     }
 
     pub fn suggest(&self, tokens: &[Token]) -> Option<String> {
         let words = tokens
             .iter()
-            .map(|token| token.as_utf8())
+            .map(|token| token.as_str())
             .collect::<Vec<_>>();
         let string = words.join(" ");
 
@@ -171,19 +179,14 @@ impl Yozuk {
     }
 }
 
+#[derive(Default)]
 pub struct YozukBuilder {
     config: Config,
     i18n: I18n,
-    logger: Logger,
     redirections: Vec<(Vec<Token>, Vec<String>)>,
 }
 
 impl YozukBuilder {
-    pub fn set_logger(mut self, logger: Logger) -> Self {
-        self.logger = logger;
-        self
-    }
-
     pub fn set_config(mut self, config: Config) -> Self {
         self.config = config;
         self
@@ -211,9 +214,7 @@ impl YozukBuilder {
     pub fn build(self, model: ModelSet) -> Yozuk {
         let build_info = concat!(r#"{"version": ""#, env!("CARGO_PKG_VERSION"), r#""}"#);
 
-        let env = Environment::new()
-            .logger(self.logger.clone())
-            .build_info(build_info);
+        let env = Environment::new().build_info(build_info);
 
         #[cfg(feature = "rayon")]
         let iter = skill::SKILLS.par_iter();
@@ -227,7 +228,7 @@ impl YozukBuilder {
                     .skills
                     .get(entry.key)
                     .zip(entry.entry.config_schema)
-                    .map(|(value, schema)| SkillConfig::new(value, schema))
+                    .map(|(value, _)| SkillConfig::new(value))
                     .unwrap_or_else(|| Ok(Default::default()))
                     .map_err(|err| (entry, err))?;
                 Ok(SkillCache {
@@ -235,13 +236,7 @@ impl YozukBuilder {
                     skill: (entry.entry.init)(&env, &config).map_err(|err| (entry, err))?,
                 })
             })
-            .collect::<Vec<_>>();
-
-        for result in &results {
-            if let Err((entry, err)) = result {
-                error!(self.logger, "Failed to initialize {}: {}", entry.key, err);
-            }
-        }
+            .collect::<Vec<Result<_, (&NamedSkillEntry, _)>>>();
 
         let mut skills = results
             .into_iter()
@@ -278,18 +273,6 @@ impl YozukBuilder {
             labelers,
             commands,
             redirections: self.redirections,
-            logger: self.logger,
-        }
-    }
-}
-
-impl Default for YozukBuilder {
-    fn default() -> Self {
-        Self {
-            config: Default::default(),
-            i18n: Default::default(),
-            logger: NullLoggerBuilder.build().unwrap(),
-            redirections: Default::default(),
         }
     }
 }
